import { format, createLogger, transports } from "winston";
import "winston-daily-rotate-file";

//Label

let alignColorsAndTime = format.combine(
  format.colorize({
    all: true,
  }),
  format.timestamp({
    format: "YY-MM-DD HH:mm:ss",
  }),
  format.printf((info) => `${info.timestamp}  ${info.level} : ${info.message}`)
);
const fileRotateTransport = new transports.DailyRotateFile({
  filename: "logs/logs-%DATE%.log",
  datePattern: "YYYY-MM-DD",
  format: format.combine(format.colorize(), alignColorsAndTime),
  maxFiles: "14d",
});
const errorFileRotateTransport = new transports.File({
  filename: "logs/error.log",
  level: "error",
});
const changeToTaskLogger = (logFile) => {
  logger.remove(fileRotateTransport);
  fileRotateTransport.filename = logFile;
  logger.add(
    new transports.File({
      filename: logFile,
      format: format.combine(format.colorize(), alignColorsAndTime),
    })
  );
};
export const logger = createLogger({
  level: "debug",
  transports: [
    new transports.Console({
      format: format.combine(format.colorize(), alignColorsAndTime),
      level: "info",
    }),
    fileRotateTransport,
    errorFileRotateTransport,
  ],
});

export { changeToTaskLogger, logger as default };
