import { Command, Types } from "./command.js";
import readline from "readline";
import logger from "../../logger.js";
import { AccessibilityDump } from "../tasks/accessibility_dump.js";

class AskUser extends Command {
  constructor(valueDescription, recording) {
    super(
      Types.ASK,
      "request infomation that is required for the next step like username password or any information that is not implicitlly available",
      [{ name: "description", description: "description of the information needed" }],
      recording
    );
    this.valueDescription = valueDescription;
    this.recordingStep.parameters = [this.valueDescription];
  }
  async run(context, executeStep = null, taskProgress) {
    this.preRun(context, taskProgress);

    taskProgress.updateLastCommand({ aiInstruction: "ask the user for: " + this.valueDescription });
    let request = "";
    if (!executeStep) {
      const rl = readline.createInterface({
        input: process.stdin,
        output: process.stdout,
      });
      logger.info(`I need the following information: ${this.valueDescription}`);
      for await (const line of rl) {
        request = line;
        break;
      }
      this.recordingStep.text = request;
    } else {
      this.valueDescription = executeStep.parameters[0];
      request = executeStep.text;
    }
    taskProgress.updateLastCommand({ success: true });

    const task2 = new AccessibilityDump();
    await task2.execute(context);

    return this.postCommand(
      `you have requested information from the user for ${this.valueDescription} and got "${request}"`,
      taskProgress,
      context
    );
  }
}
export { AskUser };
