import { ExecutePageMethod } from "./execute_page_method.js";
import { OpenEnvironment } from "./open_environment.js";
import { TaskComplete } from "./task_complete.js";
import { ClickElement } from "./click_element.js";
import { VerifyElementExist } from "./verify_element_exist.js";
import { FillElement } from "./fill_element.js";
import { AskUser } from "./ask_user.js";
import assert from "assert";
import { Types } from "./command.js";
import { RecordingTypes } from "./command.js";
//import { ClickRowAction } from "./click_row_action.js";

const getCommandFromName = function (command, recording = null) {
  switch (command.name) {
    case Types.EXECUTE:
      assert(Object.keys(command.args).length === 3);
      return new ExecutePageMethod(command.args[0], command.args[1], command.args[2], recording);
    case Types.OPEN:
      return new OpenEnvironment(recording);
    case Types.COMPLETE:
      return new TaskComplete(command.args["reason"], recording);
    case Types.CLICK:
      if (command.args && command.args["id"]) {
        return new ClickElement(parseInt(command.args["id"]), recording);
      } else {
        return new ClickElement(-1, recording);
      }
    case Types.VERIFY_ELEMENT_EXIST:
      return new VerifyElementExist(command.args["role"], command.args["name"], recording);
    case Types.FILL:
      return new FillElement(parseInt(command.args["id"]), command.args["text"], recording);
    case Types.ASK:
      return new AskUser(command.args["description"], recording);
    // case Types.CLICK_ROW_ACTION:
    //   return new ClickRowAction(parseInt(command.args["rowNumber"]), parseInt(command.args["actionNumber"]), recording);
    default:
      return new ExecutePageMethod(null, command.name, command.args, recording);
  }
};

const getCommandsSummary = function (project, operationtype = RecordingTypes.NAVIGATION) {
  const commandsText = [];
  let allCommands = [];
  if (operationtype !== RecordingTypes.FORM) {
    allCommands = project.getProjectSignature();
  }
  let commands = null;
  if (operationtype === RecordingTypes.NAVIGATION) {
    commands = [
      new ClickElement("selector"),
      new FillElement("selector", "text"),
      new VerifyElementExist("role", "name"),
      //new ClickRowAction("textInRow", "id"),
      new OpenEnvironment(),
      new AskUser("description"),
      new TaskComplete("reason"),
    ];
  } else if (operationtype === RecordingTypes.FORM) {
    commands = [
      new ClickElement("selector"),
      new FillElement("selector", "text"),
      new VerifyElementExist("role", "name"),
      //new ClickRowAction("textInRow", "id"),
      new AskUser("description"),
      new TaskComplete("reason"),
    ];
  }
  commands.forEach((command) => {
    allCommands.unshift(command.getInJsonFormat());
  });
  commandsText.push("[");
  for (let i = 0; i < allCommands.length; i++) {
    const command = allCommands[i];
    commandsText.push(`${command}`);
  }
  commandsText.push("]");
  return commandsText.join("\n");
};

export { getCommandFromName, getCommandsSummary };
