import ExecuteStep from "../tasks/execute_step.js";
import { Command, Types, postCommandSleep } from "./command.js";
import logger from "../../logger.js";
import { AccessibilityDump } from "../tasks/accessibility_dump.js";

class ExecutePageMethod extends Command {
  constructor(pageName = null, methodName, parameters = {}, recording = null) {
    super(Types.EXECUTE, null, [], recording);
    this.methodName = methodName;
    this.parameters = parameters;
    this.pageName = pageName;
    this.recordingStep.parameters = [this.pageName, this.methodName, this.parameters];
  }
  async run(context, executeStep = null, taskProgress) {
    this.preRun(context, taskProgress);

    taskProgress.updateLastCommand({
      aiInstruction: `Execute method ${this.pageName}:${this.methodName} with parameters ${JSON.stringify(
        this.parameters
      )}`,
    });

    if (executeStep) {
      this.pageName = executeStep.parameters[0];
      this.methodName = executeStep.parameters[1];
      this.parameters = executeStep.parameters[2];
    }
    let commandDescription = null;
    context.project.pages.forEach((page) => {
      let isNavigate = "navigate" === this.methodName;
      if ((isNavigate && page.getName() === this.pageName) || page.hasMethod(this.methodName)) {
        this.sourceFileName = page.sourceFileName;
        this.pageName = page.getName();
        if (isNavigate) {
          commandDescription = "navigate to " + this.pageName;
        } else {
          commandDescription = page.getMethodCommentsData(this.methodName).description;
        }
      }
    });
    if (!this.sourceFileName) {
      taskProgress.updateLastCommand({ success: false });
      taskProgress.updateLastCommand({ error: `Source file: ${this.sourceFileName} couldn't be found` });

      throw new Error(`Page ${this.pageName}.${this.methodName} not found`);
    }
    logger.info(
      `Executing method ${this.methodName} on page ${this.pageName} using parameters ${JSON.stringify(this.parameters)}`
    );
    try {
      const task = new ExecuteStep(this.sourceFileName, this.methodName, this.parameters);
      await task.execute(context);
      taskProgress.updateLastCommand({ success: true });
      await new Promise((resolve) => setTimeout(resolve, postCommandSleep));
    } catch (err) {
      taskProgress.updateLastCommand({ success: false });
      throw err;
    }

    const task2 = new AccessibilityDump();
    await task2.execute(context);

    return this.postCommand(commandDescription, taskProgress, context);
  }
}
export { ExecutePageMethod };
