import Browser from "../tasks/browser.js";

import { Command, Types } from "./command.js";
import logger from "../../logger.js";
import { AccessibilityDump } from "../tasks/accessibility_dump.js";

class OpenEnvironment extends Command {
  constructor(recording) {
    super(Types.OPEN, "open environment url in the browser", [], recording);
  }
  // eslint-disable-next-line
  async run(context, executeStep = null, taskProgress) {
    await this.preRun(context, taskProgress);
    taskProgress.updateLastCommand({ aiInstruction: "open environment url: " + context.environment.baseUrl });

    const task = new Browser(context.environment.baseUrl);

    logger.info(`Opening browser in ${context.environment.baseUrl}`);
    try {
      await task.execute(context);
    } catch (err) {
      logger.error(`Failed to open browser: ${err}`);

      taskProgress.updateLastCommand({ success: false });
      throw err;
    }

    const task2 = new AccessibilityDump();
    await task2.execute(context);

    return this.postCommand("browser opened in the environment url", taskProgress, context);
  }
}
export { OpenEnvironment };
