import { Types } from "./command.js";
import { Node } from "../tasks/accessibility_node.js";
import logger from "../../logger.js";

const _generateCodeFromStep = (step, elements) => {
  let elementsChanged = false;
  let elementIdentifier = null;
  let locatorObject = null;
  // handle element
  if (step.type === Types.CLICK || step.type === Types.FILL || step.type === Types.VERIFY_ELEMENT_EXIST) {
    let node = null;
    if (step.type === Types.VERIFY_ELEMENT_EXIST) {
      node = new Node();
      node.role = step.parameters[0];
      node.name = step.parameters[1];
      node.uniqueRoleName = true;
    } else {
      if (!step.element) {
        throw new Error(`Step ${step.type} doesn't have element`);
      }
      node = new Node();
      node.fromJson(step.element);
    }
    //locator = node.getUniqueLocatorCode();
    locatorObject = node.getUniqueLocatorObject();

    const keys = Object.keys(elements);
    keys.forEach((key) => {
      if (JSON.stringify(elements[key]) === JSON.stringify(locatorObject)) {
        elementIdentifier = key;
      }
    });
    if (!elementIdentifier) {
      elementIdentifier = (node.role + "_" + node.name).toLowerCase().replace(/[\W_]+/g, "_");
      let index = 1;
      let el = elementIdentifier;
      while (keys.includes(el)) {
        el = elementIdentifier + "_" + index;
        index++;
      }
      elementIdentifier = el;
    }
    elements[elementIdentifier] = locatorObject;
    elementsChanged = true;
  }
  const codeLines = [];
  const parameters = [];
  let line = null;
  switch (step.type) {
    case Types.CLICK:
      line = `await this.stable.click(this.elements["${elementIdentifier}"]);`;
      codeLines.push(line);
      break;
    case Types.FILL:
      line = `await this.stable.fill(this.elements["${elementIdentifier}"], `;
      // eslint-disable-next-line
      let input = `"${step.parameters[0]}"`;
      if (step.dataSource === "userData") {
        parameters.push(step.dataKey);
        input = step.dataKey;
      } else if (step.dataSource === "environment") {
        input = `this.environment["${step.dataKey}"]`;
      }
      line += `${input});`;
      codeLines.push(line);
      break;
    case Types.VERIFY_ELEMENT_EXIST:
      line = `await this.stable.verifyElementExistInPage(this.elements["${elementIdentifier}"]);`;
      codeLines.push(line);
      break;
    case Types.COMPLETE:
      break;
    default:
      logger.error(`Unknown step type ${step.type}`);
  }
  return { codeLines, elements: elementsChanged ? elements : null, parameters };
};

const generateCode = (recording, codePage) => {
  let elements = codePage.getVariableDeclarationAsObject("elements");
  //let elementsCode = codePage.getVariableDeclarationCode("elements");
  if (!elements) {
    elements = {};
  }

  const codeLines = [];
  const parameters = [];
  recording.steps.forEach((step) => {
    const result = _generateCodeFromStep(step, elements);
    codeLines.push(...result.codeLines);
    if (result.elements) {
      elements = result.elements;
    }
    if (result.parameters) {
      parameters.push(...result.parameters);
    }
  });
  return { codeLines, elements, parameters };
};
export { generateCode };
