import fs from "fs";
import path from "path";
import { dirname } from "path";
import { CodePage } from "../tasks/code/page_reflection.js";
import { generatePrompt } from "../tasks/openai/prompt_runner.js";
import { fileURLToPath } from "url";
import logger from "../../logger.js";

const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);
class Project {
  environment = null;
  pages = [];
  constructor(rootFolder) {
    this.rootFolder = rootFolder;
    this.stepsFolder = rootFolder + "/features/step_definitions";
  }

  async load() {
    this.pages = [];
    // load env.json from the project root folder
    let envFileContent = fs.readFileSync(this.rootFolder + "/env.json", "utf8");
    this.environment = JSON.parse(envFileContent);
    // list all the files in the project /features/step_definitions folder with mjs extension
    let files = fs.readdirSync(this.stepsFolder);
    for (let i = 0; i < files.length; i++) {
      let file = files[i];
      if (file.startsWith("_")) {
        continue;
      }
      if (file.endsWith(".mjs")) {
        let fullFileName =
          this.rootFolder + "/features/step_definitions/" + file;

        // create a new CodePage object for each file
        let page = new CodePage(fullFileName);
        page.generateModel();
        this.pages.push(page);
      }
    }
  }
  getPagesNames() {
    let result = [];
    this.pages.forEach((page) => {
      result.push(page.getName());
    });
    return result;
  }
  getProjectSignature() {
    let result = [];
    this.pages.forEach((page) => {
      page.getMethodsNames().forEach((methodName) => {
        result.push(page.getMethodInCommandFormat(methodName));
      });
    });
    return result;
  }
  getPage(pageName) {
    for (let i = 0; i < this.pages.length; i++) {
      if (this.pages[i].getName() === pageName) {
        return this.pages[i];
      }
    }
    return null;
  }
  createNewPage(name, _path, login_required, description, navigate_on_init) {
    let distFileName = this.stepsFolder + "/" + name + "_page.mjs";
    if (fs.existsSync(distFileName)) {
      logger.error("Page already exists: " + distFileName);
      throw new Error(
        "Page already exists: " + name + " - " + name + "_page.mjs"
      );
    }
    let map = {
      name,
      path: _path,
      login_required,
      description,
      name_upper: name.charAt(0).toUpperCase() + name.slice(1),
      navigate_on_init,
    };

    let content = generatePrompt(
      path.resolve(__dirname, "../tasks/code/page_template.txt"),
      map,
      false
    ).promptFileContent;
    fs.writeFileSync(distFileName, content, "utf8");
    this.load();
  }
}
export { Project };
