import escapeStringRegexp from "escape-string-regexp";
import reg_parser from "regex-parser";
class Node {
  constructor(axNode = null) {
    this.axNode = axNode;
    this.nodeId = -1;
    if (this.axNode) {
      this.ignored = axNode.ignored;
      this.nodeId = parseInt(axNode.nodeId);
    }
    this.focusable = false;
    this.children = [];
    this.background = false;
    this.localId = -1;
    this.locator = null;
    this.uniqueRoleName = false;
    this.parentId = -1;
    this.uniqueParentRole = null;
    this.uniqueParentName = null;
    this.href = null;
    this.pattern = null;
    this.isPatternUnique = false;
    this.nameAlternative = null;
    if (axNode && axNode.role) {
      this.roleType = axNode.role.type;
      this.role = axNode.role.value;
    }
    if (axNode && axNode.name) {
      this.nameType = axNode.name.type;
      this.name = axNode.name.value;
    }
    this.properties = null;
    if (axNode && axNode.properties && axNode.properties.length > 0) {
      this.properties = {};
      for (let i = 0; i < axNode.properties.length; i++) {
        const prop = axNode.properties[i];
        this.properties[prop.name] = prop.value.value;
      }
    }
    if (this.properties && this.properties.focusable) {
      this.focusable = this.properties.focusable;
    }
    if (this.name && /\d+$/.test(this.name)) {
      const matchEnd = this.name.match(/(\d+)$/);
      if (matchEnd) {
        const escapedInputString = escapeStringRegexp(this.name);
        this.pattern = `/^${escapedInputString.replace(matchEnd[0], "")}\\d+$/g`;
      }
    }
  }
  fromJson(json) {
    this.axNode = json.axNode;
    this.ignored = json.ignored;
    this.focusable = json.focusable;
    this.children = json.children;
    this.background = json.background;
    this.locator = json.locator;
    this.roleType = json.roleType;
    this.role = json.role;
    this.nameType = json.nameType;
    this.name = json.name;
    this.properties = json.properties;
    this.localId = json.localId;
    this.uniqueRoleName = json.uniqueRoleName;
    this.nodeId = json.nodeId;
    this.parentId = json.parentId;
    this.uniqueParentRole = json.uniqueParentRole;
    this.uniqueParentName = json.uniqueParentName;
    this.href = json.href;
    this.pattern = json.pattern;
    this.isPatternUnique = json.isPatternUnique;
    this.nameAlternative = json.nameAlternative;
  }
  // _fixNameWithNumber = (name) => {
  //   if (!name) {
  //     return name;
  //   }
  //   if (/\d+$/.test(name)) {
  //     name = escapeStringRegexp(name).replaceAll("/", "\\/");
  //     return "/" + name.replace(/\d+$/, "\\d+") + "/";
  //   } else if (/^\d+/.test(name)) {
  //     name = escapeStringRegexp(name).replaceAll("/", "\\/");
  //     return new Reg();
  //     return "/" + name.replace(/^\d+/, "\\d+") + "/";
  //   }
  //   return JSON.stringify(name);
  // };

  // escapeRegExp(string) {
  //   return string.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
  // }

  getPattern() {
    if (!this.pattern) {
      return null;
    }
    return reg_parser(this.pattern);
  }

  // getUniqueLocatorCode(exact = false) {
  //   let locator = null;
  //   let exactText = "";
  //   if (exact) {
  //     exactText = ", exact: true";
  //   }
  //   if (this.uniqueRoleName) {
  //     locator = ` [\n    { role: ["${this.role}", { name: ${this._fixNameWithNumber(this.name)}${exactText}}] }\n  ]`;
  //   } else if (this.uniqueParentRole && this.uniqueParentName) {
  //     locator = ` [\n    { role: ["${this.uniqueParentRole}", { name: ${this._fixNameWithNumber(
  //       this.uniqueParentName
  //     )}${exactText}}] },\n    { role: ["${this.role}", { name: ${this._fixNameWithNumber(this.name)}}] }\n  ]`;
  //   } else if (this.locator) {
  //     locator = `[\n    { css: ${JSON.stringify(this.locator)} }\n  ]`;
  //   } else {
  //     locator = `[\n    { text: ${JSON.stringify(this.name)} }\n  ]`;
  //   }
  //   return locator;
  // }
  getUniqueLocatorObject() {
    let locator = null;
    if (this.uniqueRoleName) {
      let roleOptions = { name: this.name };
      if (this.isPatternUnique) {
        roleOptions = { nameReg: this.pattern };
      }
      locator = [
        [
          {
            role: [this.role, roleOptions],
          },
        ],
        [{ css: this.locator }],
      ];
    } else if (this.uniqueParentRole && this.uniqueParentName) {
      let roleOptions = { name: this.name };
      if (this.isPatternUnique) {
        roleOptions = { nameReg: this.pattern };
      }
      locator = [
        [
          {
            role: [this.uniqueParentRole, { name: this.uniqueParentName }],
          },
          {
            role: [this.role, roleOptions],
          },
        ],
        [{ css: this.locator }],
      ];
    } else if (this.locator) {
      locator = [[{ css: this.locator }]];
    } else {
      locator = [[{ text: this.name }]];
    }
    return locator;
  }

  toString() {
    return `name: ${this.name ? this.name : ""}, role: ${this.role ? ", " + this.role : ""}`;
  }
  isActionable() {
    if (
      this.role === "link" ||
      this.role === "button" ||
      this.role === "tab" ||
      this.role === "menuitem" ||
      this.role === "menuitemcheckbox" ||
      this.role === "menuitemradio" ||
      this.role === "checkbox" ||
      this.role === "textbox" ||
      this.role === "combobox" ||
      this.role === "listitem"
    ) {
      return true;
    }
    return false;
  }
  getCleanProperties() {
    if (!this.properties) {
      return null;
    }
    let result = {};
    if (this.properties) {
      result = JSON.parse(JSON.stringify(this.properties));
    }
    // TODO: handle the following
    if (result.describedby) {
      delete result.describedby;
    }
    switch (this.role) {
      case "textbox":
        if (result.invalid === "false") {
          delete result.invalid;
        }
        if (result.settable === true) {
          delete result.settable;
        }
        if (result.focusable === true) {
          delete result.focusable;
        }
        if (result.readonly === false) {
          delete result.readonly;
        }
        if (result.required === false) {
          delete result.required;
        }
        if (result.multiline === false) {
          delete result.multiline;
        }
        break;
      case "button":
        if (result.invalid === "false") {
          delete result.invalid;
        }
        if (result.focusable === true) {
          delete result.focusable;
        }
        break;
      case "checkbox":
        if (result.invalid === "false") {
          delete result.invalid;
        }
        if (result.focusable === true) {
          delete result.focusable;
        }
        break;
      case "columnheader":
        if (result.required === false) {
          delete result.required;
        }
        break;
      case "gridcell":
        if (result.required === false) {
          delete result.required;
        }
        if (result.readonly === false) {
          delete result.readonly;
        }

        break;
      case "link":
        if (result.focusable === true) {
          delete result.focusable;
        }
        break;
      case "listitem":
        if (result.focusable === true) {
          delete result.focusable;
        }
    }
    if (Object.keys(result).length === 0) {
      return null;
    }
    let texts = [];
    for (const key in result) {
      texts.push(`${key}: ${JSON.stringify(result[key])}`);
    }

    return texts.join(", ");
  }

  toJSONString(idObject, nodes, tab = "\t") {
    let role = this.role;
    if (role === "StaticText") {
      role = "text";
    }
    let id = "";
    if (this.isActionable()) {
      id = "id: " + idObject.index + ", ";
      this.localId = idObject.index;
      idObject.index++;
      nodes.push(this);
    }
    let cleanProperties = this.getCleanProperties();
    if (!cleanProperties) {
      cleanProperties = "";
    } else {
      cleanProperties = ", " + cleanProperties;
    }
    let _name = this.name;
    if (!_name && this.nameAlternative) {
      _name = this.nameAlternative;
    }

    // eslint-disable-next-line
    let result = `${tab}{${id}${role}: ${_name ? JSON.stringify(_name) : '""'}${cleanProperties}`;
    if (this.children && this.children.length > 0) {
      result += ",\n";
      result += `${tab}\tchildren: [\n`;
      for (let i = 0; i < this.children.length; i++) {
        const child = this.children[i];
        result += `${child.toJSONString(idObject, nodes, tab + "\t")}${i < this.children.length - 1 ? "," : ""}\n`;
      }
      result += `${tab}\t]\n`;
      result += `${tab}}`;
    } else {
      result += "}";
    }
    return result;
  }
  getReadableType = () => {
    return this.role;
  };
  getText = () => {
    return this.name ? this.name : this.nameAlternative;
  };
}
export { Node };
