import { getContext } from "automation_model";
import { postCommandSleep } from "../commands/command.js";
import { logger } from "../../logger.js";

class Browser {
  constructor(url) {
    this.type = "browser";
    this.url = url;
  }
  async execute(context) {
    //console.log("Opening browser");
    if (context.browser === undefined) {
      const testContext = await getContext(null, false, logger);
      Object.assign(context, testContext);
      if (context.stable) {
        context.stable.logger = logger;
      }
    }
    await context.page.goto(this.url, {
      timeout: 60000,
    });
    // wait for page to load by waiting for network to be idle
    console.log("Waiting for page to load");
    await Promise.all([
      context.page.waitForLoadState("networkidle"),
      context.page.waitForLoadState("load"),
      context.page.waitForLoadState("domcontentloaded"),
    ]);
    await new Promise((resolve) => setTimeout(resolve, postCommandSleep));
  }
  async onAllTasksCompleted(context) {
    if (context.browser) {
      await context.browser.close();
      context.browser = undefined;
      context.page = undefined;
    }
  }
}
export default Browser;
