const uniqueAttrs = [
  "test-id",
  "id",
  "name",
  "placeholder",
  "alt",
  "title",
  "value",
  //    "class",
  "aria-label",
  "aria-placeholder",
  "data-testid",
  "data-cy",
  "role",
];
document.uniqueAttrs = uniqueAttrs;
const NodeType = {
  ELEMENT_NODE: 1,
  ATTRIBUTE_NODE: 2,
  TEXT_NODE: 3,
  CDATA_SECTION_NODE: 4,
  PROCESSING_INSTRUCTION_NODE: 7,
  COMMENT_NODE: 8,
  DOCUMENT_NODE: 9,
};
document.NodeType = NodeType;
function getElementInfo(element) {
  const result = {
    locator: getUniqueLocator(element, document),
    href: element.getAttribute && element.getAttribute("href") ? element.getAttribute("href") : null,
    background: false,
  };
  //console.log("result", result);
  if (!element.getBoundingClientRect) {
    return JSON.stringify(result);
  }
  let rec = element.getBoundingClientRect();
  let xCenter = rec.left + rec.width / 2;
  let yCenter = rec.top + rec.height / 2;
  let elementAtPoint = document.elementFromPoint(xCenter, yCenter);
  if (!elementAtPoint) {
    return JSON.stringify(result);
  }
  if (elementAtPoint.contains(element)) {
    return JSON.stringify(result);
  }
  // check if the element is the same element or same as parent
  let toCheck = elementAtPoint;
  while (toCheck) {
    if (toCheck === element) {
      return JSON.stringify(result);
    }
    toCheck = toCheck.parentElement;
  }
  result.background = true;
  return JSON.stringify(result);
}
document.getElementInfo = getElementInfo;
function isUniqueSelector(selector, _element, document) {
  const elements = document.querySelectorAll(selector);
  return elements.length === 1 && elements[0] === _element;
}
document.isUniqueSelector = isUniqueSelector;
function getUniqueLocator(_element, document) {
  let parent = _element;
  // find unique parent
  let unqiqueParentLocator = null;
  while (parent) {
    if (parent.hasAttribute && parent.hasAttribute("href") && parent.getAttribute("href")) {
      unqiqueParentLocator = `[href='${parent.getAttribute("href")}']`;
      break;
      // if (isUniqueSelector(unqiqueParentLocator, parent, document)) {
      //   break;
      // } else {
      //   unqiqueParentLocator = null;
      // }
    } else {
      for (const attr of document.uniqueAttrs) {
        if (parent.hasAttribute && parent.hasAttribute(attr)) {
          const value = parent.getAttribute(attr);
          if (value) {
            unqiqueParentLocator = `[${attr}='${CSS.escape(value)}']`;
            if (attr === "class") {
              unqiqueParentLocator = "." + value.trim().replace(/\s+/g, ".");
            }
            //console.log("try: ", unqiqueParentLocator);
            if (isUniqueSelector(unqiqueParentLocator, parent, document)) {
              //console.log("found unique", unqiqueParentLocator);
              break;
            } else {
              //console.log("not unique", unqiqueParentLocator);
              unqiqueParentLocator = null;
            }
          }
        }
      }
    }

    if (unqiqueParentLocator) {
      if (parent === _element) {
        return unqiqueParentLocator;
      }
      break;
    }
    parent = parent.parentElement;
  }
  if (!unqiqueParentLocator) {
    return document.fullQualifiedSelector(_element, true, parent);
  }
  let cssLocator = document.fullQualifiedSelector(_element, true, parent);
  return unqiqueParentLocator + (cssLocator ? " > " + cssLocator : "");
}
document.getUniqueLocator = getUniqueLocator;

const fullQualifiedSelector = function (node, justSelector, parent = null) {
  //console.log("fullQualifiedSelector");
  try {
    if (node.nodeType !== document.NodeType.ELEMENT_NODE) {
      //console.log("node.nodeType", node.nodeType);
      return node.localName || node.nodeName.toLowerCase();
    }
    return document.cssPath(node, justSelector, parent);
  } catch (e) {
    console.log("e", e);
    return null;
  }
};
document.fullQualifiedSelector = fullQualifiedSelector;
const cssPath = function (node, optimized, parent = null) {
  //console.log("cssPath");
  if (node.nodeType !== NodeType.ELEMENT_NODE) return "";

  const steps = [];
  let contextNode = node;
  while (contextNode && contextNode !== parent) {
    const step = document._cssPathStep(contextNode, !!optimized, contextNode === node);
    if (!step) break; // Error - bail out early.
    steps.push(step);
    if (step.optimized) break;
    contextNode = contextNode.parentNode;
  }

  steps.reverse();
  return steps.join(" > ");
};
document.cssPath = cssPath;

const _cssPathStep = function (node, optimized, isTargetNode) {
  if (node.nodeType !== NodeType.ELEMENT_NODE) return null;

  const id = node.getAttribute("id");
  if (optimized) {
    if (id) return new document.Step(idSelector(id), true);
    const nodeNameLower = node.nodeName.toLowerCase();
    if (nodeNameLower === "body" || nodeNameLower === "head" || nodeNameLower === "html")
      return new document.Step(document.nodeNameInCorrectCase(node), true);
  }
  const nodeName = document.nodeNameInCorrectCase(node);

  if (id) return new document.Step(nodeName + idSelector(id), true);
  const parent = node.parentNode;
  if (!parent || parent.nodeType === NodeType.DOCUMENT_NODE) return new document.Step(nodeName, true);

  /**
   * @param {!SDK.DOMNode} node
   * @return {!Array.<string>}
   */
  function prefixedElementClassNames(node) {
    const classAttribute = node.getAttribute("class");
    if (!classAttribute) return [];

    return classAttribute
      .split(/\s+/g)
      .filter(Boolean)
      .map(function (name) {
        // The prefix is required to store "__proto__" in a object-based map.
        return "$" + name;
      });
  }

  /**
   * @param {string} id
   * @return {string}
   */
  function idSelector(id) {
    return "#" + CSS.escape(id);
  }

  const prefixedOwnClassNamesArray = prefixedElementClassNames(node);
  let needsClassNames = false;
  let needsNthChild = false;
  let ownIndex = -1;
  let elementIndex = -1;
  const siblings = parent.children;
  for (let i = 0; (ownIndex === -1 || !needsNthChild) && i < siblings.length; ++i) {
    const sibling = siblings[i];
    if (sibling.nodeType !== NodeType.ELEMENT_NODE) continue;
    elementIndex += 1;
    if (sibling === node) {
      ownIndex = elementIndex;
      continue;
    }
    if (needsNthChild) continue;
    if (document.nodeNameInCorrectCase(sibling) !== nodeName) continue;

    needsClassNames = true;
    const ownClassNames = new Set(prefixedOwnClassNamesArray);
    if (!ownClassNames.size) {
      needsNthChild = true;
      continue;
    }
    const siblingClassNamesArray = prefixedElementClassNames(sibling);
    for (let j = 0; j < siblingClassNamesArray.length; ++j) {
      const siblingClass = siblingClassNamesArray[j];
      if (!ownClassNames.has(siblingClass)) continue;
      ownClassNames.delete(siblingClass);
      if (!ownClassNames.size) {
        needsNthChild = true;
        break;
      }
    }
  }

  let result = nodeName;
  if (
    isTargetNode &&
    nodeName.toLowerCase() === "input" &&
    node.getAttribute("type") &&
    !node.getAttribute("id") &&
    !node.getAttribute("class")
  )
    result += "[type=" + CSS.escape(node.getAttribute("type")) + "]";
  if (needsNthChild) {
    result += ":nth-child(" + (ownIndex + 1) + ")";
  } else if (needsClassNames) {
    for (const prefixedName of prefixedOwnClassNamesArray) result += "." + CSS.escape(prefixedName.slice(1));
  }

  return new document.Step(result, false);
};
document._cssPathStep = _cssPathStep;
const Step = class {
  /**
   * @param {string} value
   * @param {boolean} optimized
   */
  constructor(value, optimized) {
    this.value = value;
    this.optimized = optimized || false;
  }

  /**
   * @override
   * @return {string}
   */
  toString() {
    return this.value;
  }
};
document.Step = Step;

const nodeNameInCorrectCase = function (node) {
  const shadowRootType = node.shadowRoot && node.shadowRoot.mode;
  if (shadowRootType) return "#shadow-root (" + shadowRootType + ")";

  // If there is no local name, it's case sensitive
  if (!node.localName) return node.nodeName;

  // If the names are different lengths, there is a prefix and it's case sensitive
  if (node.localName.length !== node.nodeName.length) return node.nodeName;

  // Return the localname, which will be case insensitive if its an html node
  return node.localName;
};
document.nodeNameInCorrectCase = nodeNameInCorrectCase;
