import vm from "vm";
import { AutoPage } from "automation_model";
import { CodePage } from "./code/page_reflection.js";

class ExecuteStep {
  constructor(sourceFileName, methodName, parameters = {}) {
    this.type = "execute_step";
    this.sourceFileName = sourceFileName;
    this.methodName = methodName;
    this.parameters = parameters;
  }

  async execute(context) {
    if (!this.vmContext) {
      this.vmContext = {
        AutoPage: AutoPage,
        console: console,
        context: context,
        setTimeout: setTimeout,
      };
    }
    vm.createContext(this.vmContext); // Contextify the object.

    let codePage = new CodePage(this.sourceFileName);
    codePage.generateModel();
    let classCode = codePage.getCodeWithoutImports();
    let className = codePage.getClassName();

    let paramsValue = codePage.getMethodParametersValues(this.methodName, this.parameters);
    // check for undefined
    if (paramsValue.some((v) => v === undefined)) {
      paramsValue = [];
      Object.keys(this.parameters).forEach((k) => {
        paramsValue.push(this.parameters[k]);
      });
    }
    // convert the paramsValue array of double quoted strings delimeted by comma
    let paramsValueText = "";
    if (paramsValue.length > 0) {
      paramsValueText = paramsValue.map((v) => (v ? `"${v}"` : "null")).join(",");
    }

    let execCode = `\n(async () => {
      const v1 = await ${className}.initFromContext(context, false);
      await v1.${this.methodName}(${paramsValueText});
      await v1.stable.waitForPageLoad();
      return;
    })()`;
    const code = classCode + execCode;
    await vm.runInContext(code, this.vmContext);
    //console.log("execute step done");
    //console.log("context", context);
  }
}

export default ExecuteStep;
