function longestCommonSubstring(lines1, lines2, lines1Start, lines1Length, lines2Start, lines2Length) {
  if (!lines1 || !lines2) {
    return {
      length: 0,
      offset: 0,
      secondOffset: 0,
    };
  }

  var num = new Array(lines1Length);
  var maxlen = 0;
  var lastSubsBegin = 0;

  for (var i = 0; i < lines1Length; i++) {
    var subArray = new Array(lines2Length);
    for (var j = 0; j < lines2Length; j++) {
      subArray[j] = 0;
    }
    num[i] = subArray;
  }
  var thisSubsBegin = null;
  var secondSubsBegin = null;
  for (i = 0; i < lines1Length; i++) {
    for (j = 0; j < lines2Length; j++) {
      if (lines1[lines1Start + i].text !== lines2[lines2Start + j].text) {
        num[i][j] = 0;
      } else {
        if (i === 0 || j === 0) {
          num[i][j] = 1;
        } else {
          num[i][j] = 1 + num[i - 1][j - 1];
        }

        if (num[i][j] > maxlen) {
          maxlen = num[i][j];
          thisSubsBegin = i - num[i][j] + 1;
          secondSubsBegin = j - num[i][j] + 1;
          if (lastSubsBegin === thisSubsBegin) {
            // if the current LCS is the same as the last time this block ran
          } else {
            // this block resets the string builder if a different LCS is found
            lastSubsBegin = thisSubsBegin;
          }
        }
      }
    }
  }
  return {
    length: maxlen,
    offset: thisSubsBegin + lines1Start,
    secondOffset: secondSubsBegin + lines2Start,
  };
}
function updateCompareModel(lines1, lines2, result) {
  let unmuched = getNextUnmatchedSequence(lines1, lines2, 0, 0);
  if (unmuched.length1 === 0 || unmuched.length2 === 0) {
    return result;
  }
  const commonResult = longestCommonSubstring(
    lines1,
    lines2,
    unmuched.index1,
    unmuched.length1,
    unmuched.index2,
    unmuched.length2
  );
  let matchedCount = commonResult.length;
  result.linesMatchedCount += matchedCount;
  if (commonResult.length === 0) {
    return result;
  }
  for (let i = 0; i < matchedCount; i++) {
    const line1 = lines1[commonResult.offset + i];
    const line2 = lines2[commonResult.secondOffset + i];
    line1.matched = true;
    line1.matchedIndex = commonResult.secondOffset + i;
    line2.matched = true;
    line2.matchedIndex = commonResult.offset + i;
  }
  // befor split
  let lines1Before = [];
  let lines2Before = [];
  for (let i = 0; i < commonResult.offset; i++) {
    lines1Before.push(lines1[i]);
  }
  for (let i = 0; i < commonResult.secondOffset; i++) {
    lines2Before.push(lines2[i]);
  }
  if (lines1Before.length > 0 && lines2Before.length > 0) {
    updateCompareModel(lines1Before, lines2Before, result);
  }

  // after split
  let lines1After = [];
  let lines2After = [];
  for (let i = commonResult.offset + matchedCount; i < lines1.length; i++) {
    lines1After.push(lines1[i]);
  }
  for (let i = commonResult.secondOffset + matchedCount; i < lines2.length; i++) {
    lines2After.push(lines2[i]);
  }
  if (lines1After.length > 0 && lines2After.length > 0) {
    updateCompareModel(lines1After, lines2After, result);
  }
}
// eslint-disable-next-line
function createMatchedModel(lines1, lines2) {
  const result = {
    linesMatchedCount: 0,
    lines1Length: lines1.length,
    lines2Length: lines2.length,
  };
  updateCompareModel(lines1, lines2, result);
  return result;
}
function getNextUnmatchedSequence(lines1, lines2, index1, index2) {
  if (index1 >= lines1.length || index2 >= lines2.length) {
    return {
      index1,
      index2,
      length1: 0,
      length2: 0,
    };
  }
  // eslint-disable-next-line
  while (true) {
    if (index1 < lines1.length && lines1[index1].matched) {
      index2 = lines1[index1].matchedIndex + 1;
      index1++;
      continue;
    }
    if (index2 < lines2.length && lines2[index2].matched) {
      index1 = lines2[index2].matchedIndex + 1;
      index2++;
      continue;
    }
    let length1 = 0;
    let length2 = 0;
    for (let i = index1; i < lines1.length; i++) {
      if (lines1[i].matched) {
        break;
      }
      length1++;
    }
    for (let i = index2; i < lines2.length; i++) {
      if (lines2[i].matched) {
        break;
      }
      length2++;
    }
    return {
      index1,
      index2,
      length1,
      length2,
    };
  }
}
// let lines1 = [
//   { text: "line 1" },
//   { text: "line 2" },
//   { text: "line 3" },
//   { text: "line 4" },
//   { text: "line 5" },
//   { text: "line 6" },
//   { text: "line 7" },
//   { text: "line 8" },
// ];
// let lines2 = [
//   { text: "line 1" },
//   { text: "line 2" },
//   { text: "line 3" },
//   { text: "line y" },
//   { text: "line 4" },
//   { text: "line 5" },
//   { text: "line y" },
//   { text: "line 7" },
// ];
// console.log(createMatchedModel(lines1, lines2));
// console.log(lines1);
export { createMatchedModel };
