import path from "path";
import { dirname } from "path";
import { runPrompt } from "./prompt_runner.js";
import logger from "../../../logger.js";
import { fileURLToPath } from "url";

const __filename = fileURLToPath(import.meta.url);
const __dirname = dirname(__filename);

const isComplete = async (task, performed) => {
  const res = await runPrompt(
    path.resolve(__dirname, "prompts/is_task_complete.txt"),
    {
      task: task,
      performed: performed,
    }
    // "text-davinci-003",
    // 0.0,
    // false
  );
  logger.info("##############################################");
  logger.info(res);
  if (res.text && res.text.toLowerCase().includes("yes")) {
    return true;
  }
  return false;
};

const navigationNextCommand = async (goals, task, content, remember, environment, commands, userData) => {
  const map = {
    goals,
    task,
    content,
    remember,
    environment,
    commands,
    userData: userData ? userData : "N/A",
  };
  logger.info("page:\n" + content);
  logger.info("task:\n" + task);
  //logger.info("environment:\n" + environment);
  logger.info("commands:\n" + commands);
  logger.info("memory:\n" + remember);
  const result1 = await runPrompt(path.resolve(__dirname, "prompts/navigation.txt"), map);
  let commandObject = JSON.parse(result1.message.content);
  logger.info("##############################################");
  logger.info(commandObject);
  return commandObject;
};

const isTaskCompleteSuccessfully = async (goals, task, content, remember) => {
  const map = {
    goals,
    task,
    content,
    remember,
  };

  const result1 = await runPrompt(path.resolve(__dirname, "prompts/is_task_success.txt"), map);
  let commandObject = JSON.parse(result1.message.content);
  logger.info("##############################################");
  logger.info(commandObject);
  return commandObject;
};

const getCucumberLine = async (description, userData, type) => {
  const map = {
    description,
    userData: JSON.stringify(userData),
    type,
  };
  logger.info("data:\n" + JSON.stringify(map, null, 2));
  const result1 = await runPrompt(path.resolve(__dirname, "prompts/get_cucumber_line.txt"), map);
  let commandObject = result1.message.content;
  logger.info("##############################################");
  logger.info(commandObject);
  return commandObject;
};

const getDummyData = async (data) => {
  const map = {
    data,
  };
  logger.info("data:\n" + JSON.stringify(map, null, 2));
  const result1 = await runPrompt(path.resolve(__dirname, "prompts/generate_dummy_data.txt"), map);
  let commandObject = JSON.parse(result1.message.content);
  logger.info("##############################################");
  logger.info(commandObject);
  return commandObject;
};
const getMethodName = async (description) => {
  const map = {
    description,
  };
  logger.info("data:\n" + JSON.stringify(map, null, 2));
  const result1 = await runPrompt(path.resolve(__dirname, "prompts/description_to_method_name.txt"), map);
  let commandObject = result1.message.content;
  logger.info("##############################################");
  logger.info(commandObject);
  return commandObject;
};
export { isComplete, navigationNextCommand, getCucumberLine, getDummyData, getMethodName, isTaskCompleteSuccessfully };
