import { expect as expect } from "chai";
import { CodePage } from "../src/tasks/code/page_reflection.js";
let code = `
import { Given, When, Then, AfterStep } from "@cucumber/cucumber";

import {AutoPage} from "automation_model";

const name = "login";
const path = "/#/login";
const login_required = false;
const naviage_on_init = false;
const elements = {
  username: "input[name='username']",
  password: "input[name='password']",
  loginButton: "button[data-auto='action main']",
};
/**
 * Login page, enable UI operations on the login page
 * the main operation is login
 * @login - required
 */
class LoginPage extends AutoPage {
  constructor() {
    super();
    this.name = name;
    this.path = path;
    this.elements = elements;
    this.login_required = login_required;
    this.navigate_on_init = navigate_on_init;
  }
  /**
   * type the user name and password and click on login button in the login page
   * @param {string} username user name to use
   * @param {string} password password to use
   */
  app_login = async (username, password) => {
    console.log("login");
    this.navigate();
    await this.page.fill(this.elements.username, username);
    await this.page.fill(this.elements.password, password);
    await this.page.locator(this.elements.loginButton).hover();
    await this.page.click(this.elements.loginButton);
  };
}

Given("I'm login", async function () {
  let login = await LoginPage.init();
  await login.login("guy", "Guy12345");
});
When("I'm logout", async function () {
  let login = await LoginPage.init();
  await login.logout();
});

AfterStep(function (input) {
  //console.log("AfterStep", input);
});
`;
let file = "cucumber_demo/features/step_definitions/login_page.mjs";
describe("test add step to code", function () {
  //   it("verify that the new method command is currect", function (done) {
  //     let cp = new CodePage(file);
  //     cp.generateModel(code);
  //     cp.addNewStep(
  //       "logout",
  //       "logout from the application",
  //       {
  //         username: { type: "string", description: "user name" },
  //         password: { type: "string", description: "password" },
  //       },
  //       ['console.log("logout");', "this.navigate();"]
  //     );
  //     let expected = `logout: logout from the application, args: "username": "<user_name>", "password": "<password>"`;
  //     expect(cp.getMethodInCommandFormat("logout")).to.equal(expected);
  //     console.log(cp.getVariableDeclarationCode("elements"));
  //     done();
  //   });
  it("verify that getVariableDeclarationCode extract the code", function (done) {
    let cp = new CodePage(file);
    cp.generateModel(code);
    let expected = `const elements = {`;
    expect(cp.getVariableDeclarationCode("elements").startsWith(expected)).to.be
      .true;
    let value = cp.getVariableDeclarationAsObject("elements");
    done();
  });

  // it("2. check length is 5", function (done) {
  //   expect(numbers).to.have.lengthOf(5);
  //   done();
  // });
});
