import { getContext } from "automation_model";
import fs from "fs";
import { fileURLToPath } from "url";
import { dirname } from "path";
import path from "path";
class Node {
  constructor(axNode) {
    this.axNode = axNode;
  }
  toString() {
    return `name: ${this.name ? this.name : ""}, role: ${
      this.role ? ", " + this.role : ""
    }`;
  }
}
const buildAccessibilityTree = (root, node, nodesMap) => {
  if (!node.axNode.ignored) {
    node.ignored = false;
    if (node.axNode.role) {
      node.roleType = node.axNode.role.type;
      node.role = node.axNode.role.value;
    }
    if (node.axNode.name) {
      node.nameType = node.axNode.name.type;
      node.name = node.axNode.name.value;
    }
  }
  const childrenIds = node.axNode.childIds;
  for (let i = 0; i < childrenIds.length; i++) {
    const id = childrenIds[i];
    if (nodesMap[id]) {
      const child = new Node(nodesMap[id]);
      if (!root.children) {
        root.children = [];
      }
      if (child.axNode.ignored) {
        buildAccessibilityTree(root, child, nodesMap);
      } else {
        root.children.push(child);
        buildAccessibilityTree(node, child, nodesMap);
      }
    }
  }
};
const cleanTree = (node) => {
  let foundEmptyNodes = false;
  if (node.children) {
    for (let i = 0; i < node.children.length; i++) {
      const child = node.children[i];
      if (!child.children || child.children.length === 0) {
        if (
          child.roleType === "role" &&
          child.role === "generic" &&
          !child.name
        ) {
          node.children.splice(i, 1);
          i--;
          foundEmptyNodes = true;
        }
      }
      let result = cleanTree(child);
      if (result) {
        foundEmptyNodes = true;
      }
    }
  }
  return foundEmptyNodes;
};

const context = await getContext();
const browser = context.browser;
const page = context.page;
const playContext = context.playContext;
const cdp = await playContext.newCDPSession(page);
await cdp.send("Page.enable", {});
await cdp.send("Accessibility.enable", {});
await cdp.send("DOM.enable", {});
await cdp.send("Runtime.enable", {});
await cdp.send("CSS.enable", {});
// await cdp.on("Accessibility.loadComplete", (event) => {
//   console.log("Accessibility.loadComplete", event);
// });
await page.goto("https://github.com/login");
try {
  await Promise.all([
    page.waitForLoadState("networkidle"),
    page.waitForLoadState("load"),
    page.waitForLoadState("domcontentloaded"),
  ]);
  await new Promise((resolve) => setTimeout(resolve, 2000));
} catch (e) {
  console.log("waitForPageLoad error, ignored", e);
}
//const documentNode = await cdp.send("DOM.getDocument");
//await page.fill('input[name="q"]', "playwright");
const axNodes = await cdp.send("Accessibility.getFullAXTree", {});
const axNodeMap = {};
for (let i = 0; i < axNodes.nodes.length; i++) {
  const axNode = axNodes.nodes[i];
  axNodeMap[axNode.nodeId] = axNode;
}
const root = new Node(axNodes.nodes[0]);
buildAccessibilityTree(root, root, axNodeMap);
while (cleanTree(root)) {}
const domElements = [];
for (let i = 0; i < axNodes.nodes.length; i++) {
  const axNode = axNodes.nodes[i];
  const domNode = await cdp.send("DOM.resolveNode", {
    backendNodeId: axNode.backendDOMNodeId,
  });

  domElements.push(domNode);
  const script = "function() { return this.tagName; }";
  //console.log(targetNode);
  const res = await cdp.send("Runtime.callFunctionOn", {
    functionDeclaration: script,
    objectId: domNode.object.objectId,
  });
  console.log(res);
}

// const __filename = fileURLToPath(import.meta.url);
// const __dirname = dirname(__filename);
// const locatorBuffer = fs.readFileSync(
//   path.resolve(__dirname, "../src/tasks/browser_side/find_locator.js"),
//   "utf8"
// );
// const buffer = fs.readFileSync(
//   path.resolve(__dirname, "../src/tasks/browser_side/page_analyze.js"),
//   "utf8"
// );
// const script = locatorBuffer.toString() + "\n" + buffer.toString();
// const result1 = await cdp.send("Runtime.evaluate", {
//   expression: script,
// });

// Function to recursively search for the desired node

console.log(axNodes);
console.log(domElements);
